import * as THREE from "../build/three.module.js"
import Stats from "./jsm/libs/stats.module.js"

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0x444444);
        this._scene = scene;
        
        this._setupCamera();
        //this._setupLight();
        this._setupModel();
        //this._setupInteractions();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();
        
        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
        //.
    }

    _setupPostprocessing() {
        //.
    }

    _setupModel() {
        const radius = 1.0;
        const numLat = 100;
        const numLng = 200;
        const geometry = new THREE.BufferGeometry();
        const linePositions = new Float32Array(numLat * numLng * 3 * 2);
        const lineColors = new Float32Array(numLat * numLng * 3 * 2);
        const visible = new Float32Array(numLat * numLng * 2);

        for(let i=0; i<numLat; i++) {
            for(let j=0; j<numLng; j++) {
                const lat = (Math.random() * Math.PI) / 50.0 + i / numLat * Math.PI;
                const lng = (Math.random() * Math.PI) / 50.0 + j / numLng * 2 * Math.PI;

                const index = i * numLng + j;

                linePositions[index * 6 + 0] = 0;
                linePositions[index * 6 + 1] = 0;
                linePositions[index * 6 + 2] = 0;
                linePositions[index * 6 + 3] = radius * Math.sin(lat) * Math.cos(lng);
                linePositions[index * 6 + 4] = radius * Math.cos(lat);
                linePositions[index * 6 + 5] = radius * Math.sin(lat) * Math.sin(lng);

                const color = new THREE.Color(0xffffff);
                color.setHSL(lat / Math.PI, 1.0, 0.2);
                lineColors[index * 6 + 0] = color.r;
                lineColors[index * 6 + 1] = color.g;
                lineColors[index * 6 + 2] = color.b;

                color.setHSL(lat / Math.PI, 1.0, 0.7);
                lineColors[index * 6 + 3] = color.r;
                lineColors[index * 6 + 4] = color.g;
                lineColors[index * 6 + 5] = color.b;

                // 0 이외의 값은 보임
                visible[index * 2 + 0] = 1.0;
                visible[index * 2 + 1] = 1.0;
            }
        }

        geometry.setAttribute("position", new THREE.BufferAttribute(linePositions, 3));
        geometry.setAttribute("vertColor", new THREE.BufferAttribute(lineColors, 3));
        geometry.setAttribute("visible", new THREE.BufferAttribute(visible, 1));

        geometry.computeBoundingSphere();

        const vertexShader = `
            attribute float visible;
            varying float vVisible;
            attribute vec3 vertColor;
            varying vec3 vColor;

            void main() {
                vColor = vertColor;
                vVisible = visible;

                gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
            }
        `;

        const fragmentShader = `
            varying float vVisible;
            varying vec3 vColor;

            void main() {
                if(vVisible > 0.0) {
                    gl_FragColor = vec4(vColor, 1.0);
                } else {
                    discard;
                }
            }
        `;

        const shaderMaterial = new THREE.ShaderMaterial({vertexShader: vertexShader, fragmentShader: fragmentShader});

        const mesh = new THREE.LineSegments(geometry, shaderMaterial);
        this._scene.add(mesh);
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;
    }

    _setupCamera() {    
        const camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 0.01, 10);
        camera.position.z = 3.5;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        //.
    }

    update() {
        const time = performance.now() * 0.001;
        
        const object = this._scene.children[0];
        object.rotation.y = time * 0.5;
        object.rotation.x = time * 0.25;

        const attrVisible = object.geometry.attributes.visible;
        for(let i=0; i<attrVisible.count; i+=2) {
            if(Math.random() > 0.5) {
                attrVisible.array[i+0] = 0;
                attrVisible.array[i+1] = 0;
            } else {
                attrVisible.array[i+0] = 1;
                attrVisible.array[i+1] = 1;
            }
        }

        attrVisible.needsUpdate = true;
        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    new App();
}